/*
* $Id: enum.h,v 1.4 2003/12/17 06:04:48 jih Exp $
*
* Header file for Enum and E164 related functions
*
* Copyright (C) 2002-2003 Juha Heinanen
*
* This file is part of ser, a free SIP server.
*
* ser is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version
*
* For a license to use the ser software under conditions
* other than those described here, or to purchase support for this
* software, please contact iptel.org by e-mail at the following addresses:
* info@iptel.org
*
* ser is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#ifndef ENUM_H
#define ENUM_H
#include "../../parser/msg_parser.h"
#define MAX_DOMAIN_SIZE 256
/*
* Check if from user is an e164 number
*/
int is_from_user_e164(struct sip_msg* _msg, char* _s1, char* _s2);
/*
* Make enum query and if query succeeds, replace current uri with the
* result of the query
*/
int enum_query_0(struct sip_msg* _msg, char* _str1, char* _str2);
int enum_query_1(struct sip_msg* _msg, char* _suffix, char* _str2);
int enum_query_2(struct sip_msg* _msg, char* _suffix, char* _service);
#endif /* ENUM_H */
syntax highlighted by Code2HTML, v. 0.9.1